@@ -9,34 +9,40 @@ from bn_testing.helpers import _generate_int_suffixes
 
 class DAG(metaclass=ABCMeta):
 
-    def generate(self, n_nodes, random=None):
+    def __init__(self, n_nodes):
+        self.n_nodes = n_nodes
+        self.nodes = self._make_node_names()
+
+    def generate(self):
         raise NotImplementedError()
 
     def init(self, random):
         self.random = random
 
-    def make_node_names(self, n_nodes):
+    def _make_node_names(self):
         return _generate_int_suffixes(
             prefix='f',
-            n=n_nodes)
+            n=self.n_nodes)
 
     def show(self, dag):
         pos = nx.spring_layout(dag, seed=self.random)
         nx.draw_networkx_nodes(dag, pos=pos, node_size=100)
         nx.draw_networkx_edges(dag, pos=pos)
+        nx.draw_networkx_labels(dag, pos=pos, font_size=6)
 
 
 class ScaleFree(DAG):
 
-    def __init__(self, alpha=0.4, beta=0.5, gamma=0.1):
+    def __init__(self, n_nodes=None, alpha=0.4, beta=0.5, gamma=0.1):
+        DAG.__init__(self, n_nodes=n_nodes)
         self.alpha = alpha
         self.beta = beta
         self.gamma = gamma
 
-    def generate(self, n_nodes):
+    def generate(self):
 
         dag = nx.scale_free_graph(
-            n=n_nodes,
+            n=self.n_nodes,
             seed=self.random,
             alpha=self.alpha,
             beta=self.beta,
@@ -46,7 +52,7 @@ class ScaleFree(DAG):
 
         dag = nx.relabel_nodes(
             dag,
-            mapping=dict(zip(dag.nodes(), self.make_node_names(n_nodes)))
+            mapping=dict(zip(dag.nodes(), self.nodes))
         )
         return dag
 
@@ -58,12 +64,21 @@ class ErdosReny(DAG):
         p (float): Erdös-Renyi probability
     """
 
-    def __init__(self, p=0.1):
+    def __init__(self, n_nodes=None, p=0.1):
+        DAG.__init__(self, n_nodes=n_nodes)
         self.p = p
 
     def _select_edges(self, edges_iter, p):
         """
-        TODO
+        Creates a randomized sublist of :code:`edges_iter` where each edge is in that sublist with
+        probability :code:`p`
+
+        Args:
+            edge_iter (iterable): Iterable that yields edges
+            p (float): Selection probability
+
+        Returns:
+            list: Sublist of :code:`edges_iter`
         """
 
         # TODO Use np.fromiter
@@ -76,17 +91,15 @@ class ErdosReny(DAG):
         )
         return edges[selection]
 
-    def generate(self, n_nodes):
-
-        nodes = self.make_node_names(n_nodes)
+    def generate(self):
 
         dag = nx.DiGraph()
-        dag.add_nodes_from(nodes)
+        dag.add_nodes_from(self.nodes)
 
         # Shuffle nodes inplace
-        self.random.shuffle(nodes)
+        self.random.shuffle(self.nodes)
 
-        all_forward_edges = combinations(nodes, 2)
+        all_forward_edges = combinations(self.nodes, 2)
         edges_selected = self._select_edges(all_forward_edges, self.p)
 
         dag.add_edges_from(edges_selected)
